"""
The GUIHandler class provides GUI for Hspell, via CGI.

It processes URLs of the form http://proxpeller/...
"""

# This file is released under the terms of the GPL
# Copyright (C) Mooffie <mooffie@typo.co.il>

import cfg
import re
import cgi
import os
import templates

request_map = (
		{ 'url_ids': ["-BAD-"],
		  'template': templates.bad,
		  'filter': 0 },
		{ 'url_ids': [""],
		  'template': templates.welcome,
		  'filter': 0 },
		{ 'url_ids': ["line"],
		  'template': templates.line,
		  'filter': 0 },
		{ 'url_ids': ["textarea"],
		  'template': templates.textarea,
		  'filter': 0 },
		{ 'url_ids': ["highlight"],
		  'template': templates.highlight,
		  'filter': 1 }
	    )

bad_request = request_map[0]

gui_url_re = re.compile(r'^http://proxpeller(?:\.\w+)*/([^?]*)(?:\?(.*))?$', re.I)
results_block_re = re.compile(r'<!-- BEGIN results -->.*<!-- END results -->', re.S)

def get_request(s):
    for req in request_map:
	if s in req['url_ids']:
	    return req
    return bad_request

def utf8only_to_win1255(s):
    # convert utf8 to win1255. If the string is already win1255, keep as is.
    try:
	return unicode(s, 'utf-8').encode('iso-8859-8')
    except UnicodeError:
	return s

def parse_query_string(qs):
    query = {}
    for name, value in cgi.parse_qsl(qs):
	query[name] = utf8only_to_win1255(value)
    return query

class GUIHandler:
    def __init__(self, out, url, post_data):
	self.out     = out

	re_match = gui_url_re.search(url)
	if re_match:
	    self.request = get_request(re_match.group(1))
	    self.query   = parse_query_string(re_match.group(2) or post_data)
	else:
	    self.request = bad_request
	    self.query   = {}
	
	for field in ('text', 'hints', 'explain'):
	    self.query.setdefault(field, '')
	
	self.template = self.request['template']

    def hspell_results(self):
	args = ""
	for feature, opt in ( ('explain', '-l'),
			      ('hints',   '-n') ):
	    if self.query[feature]:
		args += " " + opt
	cmd = cfg.hspell_command % args
	wr, rd = os.popen2(cmd)
	wr.write(self.query['text'] + "\n")
	wr.close()
	return rd.read()

    def do(self):
	self.out.write("HTTP/1.0 200 OK\n")
	self.out.write("content-type: text/html\n")
	self.out.write("\n")

	# fill-out the template with the form variables...
	for field, marker in [ ('text',    '{TEXT}'),
			       ('hints',   '{B_HINTS}'),
			       ('explain', '{B_EXPLAIN}') ]:
	    val = ""
	    if self.query[field]:
		if marker.startswith('{B_'):
		    val = "checked"
		else:
		    val = cgi.escape(self.query[field], 1)
	    self.template = self.template.replace(marker, val)

	# ...and with the results
	if self.query['text']:
	    if self.template.find('{RESULTS}') != -1:
		self.template = self.template.replace('{RESULTS}',
					cgi.escape(self.hspell_results()))
	    if self.template.find('{HTMLIZED_TEXT}') != -1:
		htmlized_text = cgi.escape(self.query['text']).replace("\n", "<br />")
		self.template = self.template.replace('{HTMLIZED_TEXT}', htmlized_text)
	else:
	    self.template = results_block_re.sub('', self.template)

	# if request['filter'] is true, we pass the template through
	# htmlhandler in order to highlight misspellings
	if self.request['filter']:
	    import htmlparser
	    import htmlhandler
	    parser = htmlparser.HtmlParser(htmlhandler.HtmlHandler(self.out,
						htmlhandler.CHRST_WIN1255))
	    parser.parseString(self.template)
	else:
	    self.out.write(self.template)


