"""A parser for HTML. It resembles the SAX interface."""

# This file is released under the terms of the GPL
# Copyright (C) Mooffie <mooffie@typo.co.il>

import re

STT_IN_CDATA	= 1
STT_IN_TAG	= 2
STT_IN_COMMENT	= 3
STT_IN_PI	= 4

tag_name_re = re.compile(r'^</?([a-zA-Z]+)')
is_tag_open_close_re = re.compile(r'/\s*>$')

class HtmlContentHandler:
    def dtd(self, data):
	pass
    def startElement(self, name, tag_data):
	pass
    def endElement(self, name, tag_data):
	pass
    def characters(self, data):
	pass
    def comment(self, data):
	pass
    def processingInstruction(self, data):
	pass

class HtmlParser:
    
    def __init__(self, handler):
	self.stt = STT_IN_CDATA
	self.tag_acc = ""
	self.last_tag_is_script = 0
	self.handler = handler

    def report_cdata(self, s):
	if s:
	    self.handler.characters(s)
	    
    def report_tag(self, tag):
	if tag.startswith("<!"):
	    self.handler.dtd(tag)
	else:
	    tag_name = ""
	    res = tag_name_re.search(tag)
	    if res:
		tag_name = res.group(1).lower()
	    if tag.startswith("</"):
		self.handler.endElement(tag_name, tag)
	    else:
		self.handler.startElement(tag_name, tag)
		if is_tag_open_close_re.search(tag):
		    self.handler.endElement(tag_name, "")
		    
    def report_comment(self, s):
	self.handler.comment(s)
	
    def report_pi(self, s):
	self.handler.processingInstruction(s)

    def parseString(self, s):
	while 1:
	    if self.stt == STT_IN_CDATA:
		if self.last_tag_is_script:
		    # A SCRIPT element might contain any closing tag,
		    # and we must ignore all but </SCRIPT>
		    idx = s.lower().find("</script")
		else:
		    idx = s.find("<")
		if idx != -1:
		    self.report_cdata(s[0:idx])
		    s = s[idx:]
		    if s.startswith("<!--"):
			self.stt = STT_IN_COMMENT
		    elif s.startswith("<?"):
			self.stt = STT_IN_PI
		    else:
			self.stt = STT_IN_TAG
			self.tag_acc = ""
		else:
		    self.report_cdata(s)
		    
	    if self.stt == STT_IN_COMMENT:
		idx = s.find("-->")
		if idx != -1:
		    self.report_comment(s[0:idx+3])
		    s = s[idx+3:]
		    self.stt = STT_IN_CDATA
		    continue
		else:
		    self.report_comment(s)

	    if self.stt == STT_IN_PI:
		idx = s.find("?>")
		# I've encountered (at www.kipa.co.il) some
		# PIs (generated by Microsoft tools) that end in "/>".
		#   e.g.: <?xml:namespace ... />
		if idx == -1:
		    idx = s.find("/>")

		if idx != -1:
		    self.report_pi(s[0:idx+2])
		    s = s[idx+2:]
		    self.stt = STT_IN_CDATA
		    continue
		else:
		    self.report_pi(s)

	    if self.stt == STT_IN_TAG:
		idx = s.find(">")
		if idx != -1:
		    self.tag_acc += s[0:idx+1]
		    self.report_tag(self.tag_acc)
		    self.last_tag_is_script = self.tag_acc.lower().startswith("<script")
		    s = s[idx+1:]
		    self.stt = STT_IN_CDATA
		    continue
		else:
		    self.tag_acc += s

	    break


