#!/usr/bin/python

"""The ProxyHandler class handles an HTTP connection. It sends the request
to the ISP's proxy, and passes the filtered response back to the client"""

# This file is released under the terms of the GPL
# Copyright (C) Mooffie <mooffie@typo.co.il>

import cfg
import socket
import string
import re
import htmlparser
import htmlhandler
import guihandler
from cfg import dbgf

http_1_1_re = re.compile(r'\sHTTP/\d+\.\d+')
gui_request_re = re.compile(r'^(?:GET|POST)\s+(http://proxpeller(?:\.\w+)*(?:/\S*)?)\s', re.I)

# Remove the following headers from the request
# sent to the ISP's proxy:
excluded_request_headers = (
    "accept-encoding",	# so we won't receive "Content-encoding: gzip"
    "keep-alive",	# various "keep-alive" headers
    "connection",
    "proxy-connection"
)

class ProxyHandler:

    def __init__(self, client_in, client_out):
	self.client_in  = client_in
	self.client_out = client_out
	self.gui_url       = ""
	self.gui_post_data = ""
	s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	s.connect((cfg.isp_proxy_addr, cfg.isp_proxy_port))
	self.proxsvr = s.makefile("r+")
	s.close()
	
    def do(self):
	self.send_request();
	self.get_response_and_filter()
	self.proxsvr.close();

    def send_request(self):
	if dbgf: dbgf.write("------------REQUEST-----------\n")
	content_length = 0;
	while 1:
	    line = self.client_in.readline()
	    re_match = gui_request_re.search(line)
	    if re_match:
		self.gui_url = re_match.group(1)
	    # We don't support HTML/1.1 (most importantly, we don't support
	    # "Transfer-Encoding: chunked"), so we rewrite the request as HTTP/1.0
	    line = http_1_1_re.sub(" HTTP/1.0", line)
	    header_name = line[:line.find(':')].lower()
	    if header_name not in excluded_request_headers:
		self.proxsvr.write(line)
		if dbgf: dbgf.write(line)
	    if header_name == "content-length":
		content_length = int(line[line.find(':')+1:])
	    if not string.strip(line): break
	# Read POST data
	while content_length > 0:
	    data = self.client_in.read(content_length)
	    self.proxsvr.write(data)
	    if self.gui_url:
		self.gui_post_data += data
	    if dbgf: dbgf.write(data)
	    content_length -= len(data)
	self.proxsvr.flush()
	if dbgf: dbgf.flush()

    def get_response_and_filter(self):
	if dbgf: dbgf.write("------------RESPONSE-----------\n")
	if self.gui_url:
	    if dbgf: dbgf.write("GUI RESPONSE\n")
	    gui = guihandler.GUIHandler(self.client_out,
					self.gui_url,
					self.gui_post_data)
	    gui.do()
	    return

	is_html = 0
	charset = htmlhandler.CHRST_WIN1255
	content_length_header = ""
	while 1:
	    line = self.proxsvr.readline()
	    if not string.strip(line): break
	    header_name = line[:line.find(':')].lower()
	    if header_name == "content-length":
		content_length_header = line
	    else:
		self.client_out.write(line)
		if dbgf: dbgf.write(line)
	    if header_name == "content-type":
		if line.find("text/html") != -1:
		    is_html = 1
		    charset = htmlhandler.get_charset(line)
	# Report 'Content-Length' back if and only if it's not a
	# text/html file, because our filtering may change its size.
	if (not is_html) and content_length_header:
	    self.client_out.write(content_length_header)
	    if dbgf: dbgf.write(content_length_header)
	self.client_out.write("\n")

	if dbgf:
	    if is_html:
		dbgf.write("[HTML]\n")	
	    else:
		dbgf.write("[BINARY]\n")
	    dbgf.flush()

	if is_html:
	    parser = htmlparser.HtmlParser(htmlhandler.HtmlHandler(self.client_out, charset))
	    while 1:
		# We must not break CDATA words, so we use readline() too
		data = self.proxsvr.read(2048) + self.proxsvr.readline()
		if not data: break
		parser.parseString(data)
	else:
	    while 1:
		data = self.proxsvr.read(8192)
		if not data: break
		self.client_out.write(data)

# when started from inetd/xinetd:
if __name__ == '__main__':
    import sys
    hndlr = ProxyHandler(sys.stdin, sys.stdout)
    hndlr.do()


